;**********************************************************************
;                                                                     *
;    Filename:	    main.asm                                          *
;    Date:          2008-01-01                                        *
;    File Version:  v1.0                                              *
;                                                                     *
;    Author:        Rikard Bosnjakovic <bos@hack.org>                 *
;                                                                     *
;**********************************************************************
    list        p=12F629           ; list directive to define processor
    #include    <p12F629.inc>        ; processor specific variable definitions

    errorlevel  -302              ; suppress message 302 from list file

    __CONFIG    _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_OFF & _INTRC_OSC_NOCLKOUT 


;***** VARIABLE DEFINITIONS
; Note: START and STOP high at the same time means self-test of the lights
PIN_LEFT    equ 0                               ; Left warning light (game period 2)
PIN_RIGHT   equ 1                               ; Right warning light (game period 1 + 3)
PIN_SWITCH  equ 2                               ; Switch for selecting light (HIGH = right, LOW = left)
PIN_STOP    equ 4                               ; Stop the alarm (in case it's wrong)
PIN_START   equ 5                               ; Start the alarm

; Shared Uninitialized Data Section
            udata_shr
d1          res 1
d2          res 1
d3          res 1

;**********************************************************************
MAIN    CODE    0x000                           ; processor reset vector
        goto    start

;**********************************************************************
;                                    SUBS
;**********************************************************************
; Delay-routines from www.piclist.com
delay_1s
			;999990 cycles
	movlw	0x07
	movwf	d1
	movlw	0x2F
	movwf	d2
	movlw	0x03
	movwf	d3
delay_1s_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	$+2
	decfsz	d3, f
	goto	delay_1s_0

			;6 cycles
	goto	$+1
	goto	$+1
	goto	$+1

			;4 cycles (including call)
	return

delay_1ms
			;993 cycles
	movlw	0xC6
	movwf	d1
	movlw	0x01
	movwf	d2
delay_1ms_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	delay_1ms_0

			;3 cycles
	goto	$+1
	nop

			;4 cycles (including call)
	return


;**********************************************************************
;                           TIMER LOGIC SUBROUTINE
;**********************************************************************
start_time
        movfw   GPIO
        andlw   b'00110000'                     ; Check if both inputs are set
        xorlw   b'00110000'
        btfsc   STATUS, Z
         goto   test_mode                       ; Yep (test-mode), start both lamps

        btfsc   GPIO, PIN_SWITCH                ; Right lamp selected (switch is HIGH)?
         goto   right_light                     ; Yes, start it

left_light
        bcf     GPIO, PIN_LEFT                  ; Neither test-mode or right lamp, so
        goto    start_done                      ; we're starting the left one

test_mode
        clrf    GPIO                            ; Start both lamps
        goto    start_done

right_light
        bcf     GPIO, PIN_RIGHT                 ; Start right lamp
start_done
        call    delay_1ms                       ; Delay to ensure the port gets written to
        movlw   3                               ; Stop both lamps
        movwf   GPIO
start_debounce
        btfsc   GPIO, PIN_START                 ; If the user still presses the start-button,
         goto   start_debounce                  ; loop forever until s/he releases it
        return


;**********************************************************************
;                                    INIT
;**********************************************************************
start
        ; these first 4 instructions are not required if
        ; the internal oscillator (4MHz) is not used
        call    0x3FF                           ; retrieve factory calibration value
        bsf     STATUS,RP0                      ; set file register bank to 1 
        movwf   OSCCAL                          ; update register with factory cal value 
        bcf     STATUS,RP0                      ; set file register bank to 0

        banksel TRISIO
        movlw   b'00110100'                     ; GP[2, 4, 5] inputs, GP[0,1] outputs
        movwf   TRISIO

        banksel GPIO
        movlw   b'00000011'
        movwf   GPIO                            ; Clear garbage (GP0/1 is active high)

        banksel CMCON
        movlw   7                               ; Turn off comparators
        movwf   CMCON

        banksel OPTION_REG
        movlw   b'10000000'                     ; Disable GPIO-pullups
        movwf   OPTION_REG

        clrf    INTCON                          ; Disable all interrupts

;**********************************************************************
;                                    MAIN
;**********************************************************************
        banksel GPIO
main
        btfsc   GPIO, PIN_START
         call   start_time

        goto    main

		END                       ; directive 'end of program'
